#if !defined(COMPLEXCW_H)
#define COMPLEXCW_H

#if defined(HAVE_COMPLEX)

#include <deccw.h>
#include <complex.h>

#if !defined(I)

#if defined(_Imaginary_I)
#define I	_Imaginary_I
#elif defined(_Complex_I)
#define I	_Complex_I
#elif defined(__GNUC__)
#define I	(1.0iF)
#endif

#endif /* !defined(I) */

#undef CMPLX
#define CMPLX(x,y) ((x) + I * (y)) /* constructor (like Fortran cmplx(x,y)) */

/*
** NB: Parenthesized function names are ESSENTIAL here to avoid
** erroneous macro expansion on some platforms (e.g., <complex.h> in
** FreeBSD 5.x defines cimag*() and creal*() as macros that break
** prototypes with unparenthesized function names).
*/

extern int			(iscinff) (float complex z);
extern int			(iscnanf) (float complex z);

extern float			(cabsf) (float complex z);
extern float			(cargf) (float complex z);
extern float			(cimagf) (float complex z);
extern float			(crealf) (float complex z);
extern float complex		(cacosf) (float complex z);
extern float complex		(cacoshf) (float complex z);
extern float complex		(caddf) (float complex x, float complex y);
extern float complex		(casinf) (float complex z);
extern float complex		(casinhf) (float complex z);
extern float complex		(catanf) (float complex z);
extern float complex		(catanhf) (float complex z);
extern float complex		(ccbrtf) (float complex z);
extern float complex		(ccopyf) (float complex z);
extern float complex		(ccosf) (float complex z);
extern float complex		(ccoshf) (float complex z);
extern float complex		(cdivf) (float complex x, float complex y);
extern float complex		(cexpf) (float complex z);
extern float complex		(cexpm1f) (float complex z);
extern float complex		(cipowf) (float complex z, int n);
extern float complex		(clogf) (float complex z);
extern float complex		(clog1pf) (float complex z);
extern float complex		(cmulf) (float complex x, float complex y);
extern float complex		(cnegf) (float complex z);
extern float complex		(conjf) (float complex z);
extern float complex		(cpowf) (float complex z, float complex a);
extern float complex		(cprojf) (float complex z);
extern float complex		(csinf) (float complex z);
extern float complex		(csinhf) (float complex z);
extern float complex		(csqrtf) (float complex z);
extern float complex		(csubf) (float complex x, float complex y);
extern float complex		(ctanf) (float complex z);
extern float complex		(ctanhf) (float complex z);

extern int			(iscinf) (double complex z);
extern int			(iscnan) (double complex z);

extern double			(cabs) (double complex z);
extern double			(carg) (double complex z);
extern double			(cimag) (double complex z);
extern double			(creal) (double complex z);
extern double complex		(cacos) (double complex z);
extern double complex		(cacosh) (double complex z);
extern double complex		(cadd) (double complex x, double complex y);
extern double complex		(casin) (double complex z);
extern double complex		(casinh) (double complex z);
extern double complex		(catan) (double complex z);
extern double complex		(catanh) (double complex z);
extern double complex		(ccbrt) (double complex z);
extern double complex		(ccopy) (double complex z);
extern double complex		(ccos) (double complex z);
extern double complex		(ccosh) (double complex z);
extern double complex		(cdiv) (double complex x, double complex y);
extern double complex		(cexp) (double complex z);
extern double complex		(cexpm1) (double complex z);
extern double complex		(cipow) (double complex z, int n);
extern double complex		(clog) (double complex z);
extern double complex		(clog1p) (double complex z);
extern double complex		(cmul) (double complex x, double complex y);
extern double complex		(cneg) (double complex z);
extern double complex		(conj) (double complex z);
extern double complex		(cpow) (double complex z, double complex a);
extern double complex		(cproj) (double complex z);
extern double complex		(csin) (double complex z);
extern double complex		(csinh) (double complex z);
extern double complex		(csqrt) (double complex z);
extern double complex		(csub) (double complex x, double complex y);
extern double complex		(ctan) (double complex z);
extern double complex		(ctanh) (double complex z);

extern int			(iscinfl) (long double complex z);
extern int			(iscnanl) (long double complex z);

extern long double		(cabsl) (long double complex z);
extern long double		(cargl) (long double complex z);
extern long double		(cimagl) (long double complex z);
extern long double		(creall) (long double complex z);
extern long double complex	(cacoshl) (long double complex z);
extern long double complex	(cacosl) (long double complex z);
extern long double complex	(caddl) (long double complex x, long double complex y);
extern long double complex	(casinhl) (long double complex z);
extern long double complex	(casinl) (long double complex z);
extern long double complex	(catanhl) (long double complex z);
extern long double complex	(ccbrtl) (long double complex z);
extern long double complex	(catanl) (long double complex z);
extern long double complex	(ccopyl) (long double complex z);
extern long double complex	(ccoshl) (long double complex z);
extern long double complex	(ccosl) (long double complex z);
extern long double complex	(cdivl) (long double complex x, long double complex y);
extern long double complex	(cexpl) (long double complex z);
extern long double complex	(cexpm1l) (long double complex z);
extern long double complex	(cipowl) (long double complex z, int n);
extern long double complex	(clogl) (long double complex z);
extern long double complex	(clog1pl) (long double complex z);
extern long double complex	(cmull) (long double complex x, long double complex y);
extern long double complex	(cnegl) (long double complex z);
extern long double complex	(conjl) (long double complex z);
extern long double complex	(cpowl) (long double complex z, long double complex a);
extern long double complex	(cprojl) (long double complex z);
extern long double complex	(csinhl) (long double complex z);
extern long double complex	(csinl) (long double complex z);
extern long double complex	(csqrtl) (long double complex z);
extern long double complex	(csubl) (long double complex x, long double complex y);
extern long double complex	(ctanhl) (long double complex z);
extern long double complex	(ctanl) (long double complex z);

#if defined(HAVE_LONG_LONG_DOUBLE)

#if 0 && defined(__GNUC__)
#define long_long_double_complex	long double complex
#else
typedef long double complex long_long_double_complex; /* TO DO: update when 256-bit types available */
#endif

extern int			(iscinfll) (long_long_double_complex z);
extern int			(iscnanll) (long_long_double_complex z);

extern long_long_double		(cabsll) (long_long_double_complex z);
extern long_long_double		(cargll) (long_long_double_complex z);
extern long_long_double		(cimagll) (long_long_double_complex z);
extern long_long_double		(crealll) (long_long_double_complex z);
extern long_long_double_complex	(cacoshll) (long_long_double_complex z);
extern long_long_double_complex	(cacosll) (long_long_double_complex z);
extern long_long_double_complex	(caddll) (long_long_double_complex x, long_long_double_complex y);
extern long_long_double_complex	(casinhll) (long_long_double_complex z);
extern long_long_double_complex	(casinll) (long_long_double_complex z);
extern long_long_double_complex	(catanhll) (long_long_double_complex z);
extern long_long_double_complex	(ccbrtll) (long_long_double_complex z);
extern long_long_double_complex	(catanll) (long_long_double_complex z);
extern long_long_double_complex	(ccopyll) (long_long_double_complex z);
extern long_long_double_complex	(ccoshll) (long_long_double_complex z);
extern long_long_double_complex	(ccosll) (long_long_double_complex z);
extern long_long_double_complex	(cdivll) (long_long_double_complex x, long_long_double_complex y);
extern long_long_double_complex	(cexpll) (long_long_double_complex z);
extern long_long_double_complex	(cexpm1ll) (long_long_double_complex z);
extern long_long_double_complex	(cipowll) (long_long_double_complex z, int n);
extern long_long_double_complex	(clogll) (long_long_double_complex z);
extern long_long_double_complex	(clog1pll) (long_long_double_complex z);
extern long_long_double_complex	(cmulll) (long_long_double_complex x, long_long_double_complex y);
extern long_long_double_complex	(cnegll) (long_long_double_complex z);
extern long_long_double_complex	(conjll) (long_long_double_complex z);
extern long_long_double_complex	(cpowll) (long_long_double_complex z, long_long_double_complex a);
extern long_long_double_complex	(cprojll) (long_long_double_complex z);
extern long_long_double_complex	(csinhll) (long_long_double_complex z);
extern long_long_double_complex	(csinll) (long_long_double_complex z);
extern long_long_double_complex	(csqrtll) (long_long_double_complex z);
extern long_long_double_complex	(csubll) (long_long_double_complex x, long_long_double_complex y);
extern long_long_double_complex	(ctanhll) (long_long_double_complex z);
extern long_long_double_complex	(ctanll) (long_long_double_complex z);

#endif	/* defined(HAVE_LONG_LONG_DOUBLE) */

#if defined(__ia64) && defined(__hpux) && defined(_FPWIDETYPES)

/* NB: _FPWIDETYPES is set by the HP-UX IA-64 compilers in response to
   the -fpwidetypes command-line option */

/* Mimic the HP-UX practice in <float.h>, <math.h>, and <stdlib.h> of
   providing typedefs of the user types extended and quad if they are
   not yet defined. */

#ifndef _INCLUDE_QUAD
typedef __float128 quad;
#define _INCLUDE_QUAD
#endif

#ifndef _INCLUDE_EXTENDED
typedef __float80 extended;
#define _INCLUDE_EXTENDED
#endif

extern int			(iscinfw) (__float80 complex z);
extern int			(iscnanw) (__float80 complex z);

extern __float80		(cabsw) (__float80 complex z);
extern __float80		(cargw) (__float80 complex z);
extern __float80		(cimagw) (__float80 complex z);
extern __float80		(crealw) (__float80 complex z);
extern __float80 complex	(cacosw) (__float80 complex z);
extern __float80 complex	(cacoshw) (__float80 complex z);
extern __float80 complex	(caddw) (__float80 complex x, __float80 complex y);
extern __float80 complex	(casinw) (__float80 complex z);
extern __float80 complex	(casinhw) (__float80 complex z);
extern __float80 complex	(catanw) (__float80 complex z);
extern __float80 complex	(catanhw) (__float80 complex z);
extern __float80 complex	(ccbrtw) (__float80 complex z);
extern __float80 complex	(ccopyw) (__float80 complex z);
extern __float80 complex	(ccosw) (__float80 complex z);
extern __float80 complex	(ccoshw) (__float80 complex z);
extern __float80 complex	(cdivw) (__float80 complex x, __float80 complex y);
extern __float80 complex	(cexpw) (__float80 complex z);
extern __float80 complex	(cexpm1w) (__float80 complex z);
extern __float80 complex	(cipoww) (__float80 complex z, int n);
extern __float80 complex	(clogw) (__float80 complex z);
extern __float80 complex	(clog1pw) (__float80 complex z);
extern __float80 complex	(cmulw) (__float80 complex x, __float80 complex y);
extern __float80 complex	(cnegw) (__float80 complex z);
extern __float80 complex	(conjw) (__float80 complex z);
extern __float80 complex	(cpoww) (__float80 complex z, __float80 complex a);
extern __float80 complex	(cprojw) (__float80 complex z);
extern __float80 complex	(csinw) (__float80 complex z);
extern __float80 complex	(csinhw) (__float80 complex z);
extern __float80 complex	(csqrtw) (__float80 complex z);
extern __float80 complex	(csubw) (__float80 complex x, __float80 complex y);
extern __float80 complex	(ctanw) (__float80 complex z);
extern __float80 complex	(ctanhw) (__float80 complex z);

extern int			(iscinfq) (__float128 complex z);
extern int			(iscnanq) (__float128 complex z);

extern __float128		(cabsq) (__float128 complex z);
extern __float128		(cargq) (__float128 complex z);
extern __float128		(cimagq) (__float128 complex z);
extern __float128		(crealq) (__float128 complex z);
extern __float128 complex	(cacosq) (__float128 complex z);
extern __float128 complex	(cacoshq) (__float128 complex z);
extern __float128 complex	(caddq) (__float128 complex x, __float128 complex y);
extern __float128 complex	(casinq) (__float128 complex z);
extern __float128 complex	(casinhq) (__float128 complex z);
extern __float128 complex	(catanq) (__float128 complex z);
extern __float128 complex	(catanhq) (__float128 complex z);
extern __float128 complex	(ccbrtq) (__float128 complex z);
extern __float128 complex	(ccopyq) (__float128 complex z);
extern __float128 complex	(ccosq) (__float128 complex z);
extern __float128 complex	(ccoshq) (__float128 complex z);
extern __float128 complex	(cdivq) (__float128 complex x, __float128 complex y);
extern __float128 complex	(cexpq) (__float128 complex z);
extern __float128 complex	(cexpm1q) (__float128 complex z);
extern __float128 complex	(cipowq) (__float128 complex z, int n);
extern __float128 complex	(clogq) (__float128 complex z);
extern __float128 complex	(clog1pq) (__float128 complex z);
extern __float128 complex	(cmulq) (__float128 complex x, __float128 complex y);
extern __float128 complex	(cnegq) (__float128 complex z);
extern __float128 complex	(conjq) (__float128 complex z);
extern __float128 complex	(cpowq) (__float128 complex z, __float128 complex a);
extern __float128 complex	(cprojq) (__float128 complex z);
extern __float128 complex	(csinq) (__float128 complex z);
extern __float128 complex	(csinhq) (__float128 complex z);
extern __float128 complex	(csqrtq) (__float128 complex z);
extern __float128 complex	(csubq) (__float128 complex x, __float128 complex y);
extern __float128 complex	(ctanq) (__float128 complex z);
extern __float128 complex	(ctanhq) (__float128 complex z);

#endif /* defined(__ia64) && defined(__hpux) && defined(_FPWIDETYPES) */

#if 0 && defined(__GNUC__)
	/* the complex decimal types require compiler extensions on some systems */
extern int			(iscinfdf) (decimal_float complex z);
extern int			(iscnandf) (decimal_float complex z);

extern decimal_float		(cabsdf) (decimal_float complex z);
extern decimal_float		(cargdf) (decimal_float complex z);
extern decimal_float		(cimagdf) (decimal_float complex z);
extern decimal_float		(crealdf) (decimal_float complex z);
extern decimal_float complex	(cacosdf) (decimal_float complex z);
extern decimal_float complex	(cacoshdf) (decimal_float complex z);
extern decimal_float complex	(cadddf) (decimal_float complex x, decimal_float complex y);
extern decimal_float complex	(casindf) (decimal_float complex z);
extern decimal_float complex	(casinhdf) (decimal_float complex z);
extern decimal_float complex	(catandf) (decimal_float complex z);
extern decimal_float complex	(catanhdf) (decimal_float complex z);
extern decimal_float complex	(ccbrtdf) (decimal_float complex z);
extern decimal_float complex	(ccopydf) (decimal_float complex z);
extern decimal_float complex	(ccosdf) (decimal_float complex z);
extern decimal_float complex	(ccoshdf) (decimal_float complex z);
extern decimal_float complex	(cdivdf) (decimal_float complex x, decimal_float complex y);
extern decimal_float complex	(cexpdf) (decimal_float complex z);
extern decimal_float complex	(cexpm1df) (decimal_float complex z);
extern decimal_float complex	(cipowdf) (decimal_float complex z, int n);
extern decimal_float complex	(clogdf) (decimal_float complex z);
extern decimal_float complex	(clog1pdf) (decimal_float complex z);
extern decimal_float complex	(cmuldf) (decimal_float complex x, decimal_float complex y);
extern decimal_float complex	(cnegdf) (decimal_float complex z);
extern decimal_float complex	(conjdf) (decimal_float complex z);
extern decimal_float complex	(cpowdf) (decimal_float complex z, decimal_float complex a);
extern decimal_float complex	(cprojdf) (decimal_float complex z);
extern decimal_float complex	(csindf) (decimal_float complex z);
extern decimal_float complex	(csinhdf) (decimal_float complex z);
extern decimal_float complex	(csqrtdf) (decimal_float complex z);
extern decimal_float complex	(csubdf) (decimal_float complex x, decimal_float complex y);
extern decimal_float complex	(ctandf) (decimal_float complex z);
extern decimal_float complex	(ctanhdf) (decimal_float complex z);

extern int			(iscinfd) (decimal_double complex z);
extern int			(iscnand) (decimal_double complex z);

extern decimal_double		(cabsd) (decimal_double complex z);
extern decimal_double		(cargd) (decimal_double complex z);
extern decimal_double		(cimagd) (decimal_double complex z);
extern decimal_double		(creald) (decimal_double complex z);
extern decimal_double complex	(cacosd) (decimal_double complex z);
extern decimal_double complex	(cacoshd) (decimal_double complex z);
extern decimal_double complex	(caddd) (decimal_double complex x, decimal_double complex y);
extern decimal_double complex	(casind) (decimal_double complex z);
extern decimal_double complex	(casinhd) (decimal_double complex z);
extern decimal_double complex	(catand) (decimal_double complex z);
extern decimal_double complex	(catanhd) (decimal_double complex z);
extern decimal_double complex	(ccbrtd) (decimal_double complex z);
extern decimal_double complex	(ccopyd) (decimal_double complex z);
extern decimal_double complex	(ccosd) (decimal_double complex z);
extern decimal_double complex	(ccoshd) (decimal_double complex z);
extern decimal_double complex	(cdivd) (decimal_double complex x, decimal_double complex y);
extern decimal_double complex	(cexpd) (decimal_double complex z);
extern decimal_double complex	(cexpm1d) (decimal_double complex z);
extern decimal_double complex	(cipowd) (decimal_double complex z, int n);
extern decimal_double complex	(clogd) (decimal_double complex z);
extern decimal_double complex	(clog1pd) (decimal_double complex z);
extern decimal_double complex	(cmuld) (decimal_double complex x, decimal_double complex y);
extern decimal_double complex	(cnegd) (decimal_double complex z);
extern decimal_double complex	(conjd) (decimal_double complex z);
extern decimal_double complex	(cpowd) (decimal_double complex z, decimal_double complex a);
extern decimal_double complex	(cprojd) (decimal_double complex z);
extern decimal_double complex	(csind) (decimal_double complex z);
extern decimal_double complex	(csinhd) (decimal_double complex z);
extern decimal_double complex	(csqrtd) (decimal_double complex z);
extern decimal_double complex	(csubd) (decimal_double complex x, decimal_double complex y);
extern decimal_double complex	(ctand) (decimal_double complex z);
extern decimal_double complex	(ctanhd) (decimal_double complex z);

#if defined(HAVE_DECIMAL_LONG_DOUBLE)

extern int				(iscinfdl) (decimal_long_double complex z);
extern int				(iscnandl) (decimal_long_double complex z);

extern decimal_long_double		(cabsdl) (decimal_long_double complex z);
extern decimal_long_double		(cargdl) (decimal_long_double complex z);
extern decimal_long_double		(cimagdl) (decimal_long_double complex z);
extern decimal_long_double		(crealdl) (decimal_long_double complex z);
extern decimal_long_double complex	(cacosdl) (decimal_long_double complex z);
extern decimal_long_double complex	(cacoshdl) (decimal_long_double complex z);
extern decimal_long_double complex	(cadddl) (decimal_long_double complex x, decimal_long_double complex y);
extern decimal_long_double complex	(casindl) (decimal_long_double complex z);
extern decimal_long_double complex	(casinhdl) (decimal_long_double complex z);
extern decimal_long_double complex	(catandl) (decimal_long_double complex z);
extern decimal_long_double complex	(catanhdl) (decimal_long_double complex z);
extern decimal_long_double complex	(ccbrtdl) (decimal_long_double complex z);
extern decimal_long_double complex	(ccopydl) (decimal_long_double complex z);
extern decimal_long_double complex	(ccosdl) (decimal_long_double complex z);
extern decimal_long_double complex	(ccoshdl) (decimal_long_double complex z);
extern decimal_long_double complex	(cdivdl) (decimal_long_double complex x, decimal_long_double complex y);
extern decimal_long_double complex	(cexpdl) (decimal_long_double complex z);
extern decimal_long_double complex	(cexpm1dl) (decimal_long_double complex z);
extern decimal_long_double complex	(cipowdl) (decimal_long_double complex z, int n);
extern decimal_long_double complex	(clogdl) (decimal_long_double complex z);
extern decimal_long_double complex	(clog1pdl) (decimal_long_double complex z);
extern decimal_long_double complex	(cmuldl) (decimal_long_double complex x, decimal_long_double complex y);
extern decimal_long_double complex	(cnegdl) (decimal_long_double complex z);
extern decimal_long_double complex	(conjdl) (decimal_long_double complex z);
extern decimal_long_double complex	(cpowdl) (decimal_long_double complex z, decimal_long_double complex a);
extern decimal_long_double complex	(cprojdl) (decimal_long_double complex z);
extern decimal_long_double complex	(csindl) (decimal_long_double complex z);
extern decimal_long_double complex	(csinhdl) (decimal_long_double complex z);
extern decimal_long_double complex	(csqrtdl) (decimal_long_double complex z);
extern decimal_long_double complex	(csubdl) (decimal_long_double complex x, decimal_long_double complex y);
extern decimal_long_double complex	(ctandl) (decimal_long_double complex z);
extern decimal_long_double complex	(ctanhdl) (decimal_long_double complex z);

#endif /* defined(HAVE_DECIMAL_LONG_DOUBLE) */

#if defined(HAVE_DECIMAL_LONG_LONG_DOUBLE)

extern int				(iscinfdll) (decimal_long_long_double complex z);
extern int				(iscnandll) (decimal_long_long_double complex z);

extern decimal_long_long_double		(cabsdll) (decimal_long_long_double complex z);
extern decimal_long_long_double		(cargdll) (decimal_long_long_double complex z);
extern decimal_long_long_double		(cimagdll) (decimal_long_long_double complex z);
extern decimal_long_long_double		(crealdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(cacosdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(cacoshdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(cadddll) (decimal_long_long_double complex x, decimal_long_long_double complex y);
extern decimal_long_long_double complex	(casindll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(casinhdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(catandll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(catanhdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(ccbrtdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(ccopydll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(ccosdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(ccoshdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(cdivdll) (decimal_long_long_double complex x, decimal_long_long_double complex y);
extern decimal_long_long_double complex	(cexpdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(cexpm1dll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(cipowdll) (decimal_long_long_double complex z, int n);
extern decimal_long_long_double complex	(clogdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(clog1pdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(cmuldll) (decimal_long_long_double complex x, decimal_long_long_double complex y);
extern decimal_long_long_double complex	(cnegdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(conjdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(cpowdll) (decimal_long_long_double complex z, decimal_long_long_double complex a);
extern decimal_long_long_double complex	(cprojdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(csindll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(csinhdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(csqrtdll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(csubdll) (decimal_long_long_double complex x, decimal_long_long_double complex y);
extern decimal_long_long_double complex	(ctandll) (decimal_long_long_double complex z);
extern decimal_long_long_double complex	(ctanhdll) (decimal_long_long_double complex z);

#endif /* defined(HAVE_DECIMAL_LONG_LONG_DOUBLE) */

#endif /* 0 && defined(__GNUC__) */

#if defined(CXCW_H)

#define CTOCX_(result,z)		CXSET_(result, CREAL(z), CIMAG(z))

/* Prototypes for conversion between complex-as-real and native complex */

extern float complex 			(cxtocf)	(cx_float);
extern double complex 			(cxtoc)		(cx_double);
extern long double complex 		(cxtocl)	(cx_long_double);

#if defined(HAVE_FP_T_EXTENDED)
extern __float80 complex 		(cxtocw)	(cx_float80);
#endif /* defined(HAVE_FP_T_EXTENDED) */

#if defined(HAVE_FP_T_QUAD)
extern __float128 complex 		(cxtocq)	(cx_float128);
#endif /* defined(HAVE_FP_T_QUAD) */

extern long double complex 		(cxtocll)	(cx_long_long_double);

#if defined(HAVE_DECIMAL_COMPLEX)

extern decimal_float complex 		(cxtocdf)	(cx_decimal_float);
extern decimal_double complex 		(cxtocd)	(cx_decimal_double);

#if defined(HAVE_DECIMAL_LONG_DOUBLE)
extern decimal_long_double complex 	(cxtocdl)	(cx_decimal_long_double);
#endif /* defined(HAVE_DECIMAL_LONG_DOUBLE) */

#if defined(HAVE_DECIMAL_LONG_LONG_DOUBLE)
extern decimal_long_long_double complex (cxtocdll)	(cx_decimal_long_long_double);
#endif /* defined(HAVE_DECIMAL_LONG_LONG_DOUBLE) */

#endif /* defined(HAVE_DECIMAL_COMPLEX) */

extern void (ctocxf)	(cx_float, float complex);
extern void (ctocx)	(cx_double, double complex);
extern void (ctocxl)	(cx_long_double, long double complex);

#if defined(HAVE_FP_T_EXTENDED)
extern void (ctocxw)	(cx_float80, float80 complex);
#endif /* defined(HAVE_FP_T_EXTENDED) */

#if defined(HAVE_FP_T_QUAD)
extern void (ctocxq)	(cx_float128, float128 complex);
#endif /* defined(HAVE_FP_T_QUAD) */

extern void (ctocxll)	(cx_long_long_double, long double complex);

#if defined(HAVE_DECIMAL_COMPLEX)

extern void (ctocxdf)	(cx_decimal_float, decimal_float complex);
extern void (ctocxd)	(cx_decimal_double, decimal_double complex);

#if defined(HAVE_DECIMAL_LONG_DOUBLE)
extern void (ctocxdl)	(cx_decimal_long_double, decimal_long_double complex);
#endif /* defined(HAVE_DECIMAL_LONG_DOUBLE) */

#if defined(HAVE_DECIMAL_LONG_LONG_DOUBLE)
extern void (ctocxdll)	(cx_decimal_long_long_double, decimal_long_long_double complex);
#endif /* defined(HAVE_DECIMAL_LONG_LONG_DOUBLE) */

#endif /* defined(HAVE_DECIMAL_COMPLEX) */

#endif /* defined(CXCW_H) */

#endif /* defined(HAVE_COMPLEX) */

#endif /* !defined(COMPLEXCW_H) */
